from typing import Any, Dict, Optional

import requests

from .models import (
    APIServer,
    BooleanResponse,
    CartPaymentRefundResponse,
    CreateUpdateCartResponse,
    GetCartOrdersResponse,
    GetCartResponse,
    GetCartsResponse,
    ProcessCartResponse,
    SearchDepartmentsResponse,
    SearchGLAccountsResponse,
    SearchPaymentMethodsResponse,
    SearchPaymentsResponse,
    SearchStoresResponse
)


class MsbPayAPI:
    def __init__(self, authorization: str, server: APIServer) -> None:
        """
        :param authorization: Example: demoaccount:EBNF-JNEC-HKHL-EALD-JMFJ-BKOM-PHNA-CKLP
        :param server: The base URL for the MSB Pay API
        """
        self.server = server
        self.session = requests.Session()
        self.session.headers['Authorization'] = authorization

    def _get(self, endpoint: str, params: Dict[str, Any] = {}) -> Any:
        response = self.session.get(
            self.server.value + endpoint, params=params)
        response.raise_for_status()

        json = response.json()
        return json

    def _post(self, endpoint: str, params: Dict[str, Any] = {}, data: Optional[Dict[str, Any]] = None) -> Any:
        response = self.session.post(
            self.server.value + endpoint, params=params, json=data)
        response.raise_for_status()

        json = response.json()
        return json

    def _put(self, endpoint: str, params: Dict[str, Any] = {}, data: Optional[Dict[str, Any]] = None) -> Any:
        response = self.session.put(
            self.server.value + endpoint, params=params, json=data)
        response.raise_for_status()

        json = response.json()
        return json

    def _delete(self, endpoint: str, params: Dict[str, Any] = {}) -> Any:
        response = self.session.delete(
            self.server.value + endpoint, params=params)
        response.raise_for_status()

        json = response.json()
        return json

    def get_system_details(self):
        return self._get('/')

    def get_identity(self) -> Dict[str, Any]:
        return self._get('/whoAmI')

    def get_carts(self, **kwargs: Any) -> GetCartsResponse:
        return self._get('/carts', kwargs)

    def create_cart(self, data: Dict[str, Any]) -> CreateUpdateCartResponse:
        return self._post('/carts', data=data)

    def get_cart(self, cart_id: str) -> GetCartResponse:
        return self._get(f'/carts/{cart_id}')

    def update_cart(self, cart_id: str, data: Dict[str, Any]) -> CreateUpdateCartResponse:
        return self._put(f'/carts/{cart_id}', data=data)

    def delete_cart(self, cart_id: str) -> BooleanResponse:
        return self._delete(f'/carts/{cart_id}')

    def add_items_to_cart(self, cart_id: str, data: Dict[str, Any]) -> CreateUpdateCartResponse:
        return self._post(f'/carts/{cart_id}/addItems', data=data)

    def process_cart(self, cart_id: str) -> ProcessCartResponse:
        return self._post(f'/carts/{cart_id}/process')

    def get_cart_orders(self, cart_id: str) -> GetCartOrdersResponse:
        return self._get(f'/carts/{cart_id}/orders')

    def get_cart_payments(self, cart_id: str) -> SearchPaymentsResponse:
        return self._get(f'/carts/{cart_id}/payments')

    def refund(self, cart_id: str, payment_id: str, data: Dict[str, Any]) -> CartPaymentRefundResponse:
        return self._post(f'/carts/{cart_id}/payments/{payment_id}/refund', data=data)

    def get_payments(self, client_id: str, **kwargs: Any) -> SearchPaymentsResponse:
        return self._get(f'/clients/{client_id}/payments', kwargs)

    def confirm_transfer(self, client_id: str, payment_id: str) -> BooleanResponse:
        return self._post(f'/clients/{client_id}/payments/{payment_id}/confirmtransfer')

    def get_payment_methods(self, client_id: str, **kwargs: Any) -> SearchPaymentMethodsResponse:
        return self._get(f'/clients/{client_id}/paymentMethods', kwargs)

    def get_stores(self, client_id: str, **kwargs: Any) -> SearchStoresResponse:
        return self._get(f'/clients/{client_id}/stores', kwargs)

    def get_departments(self, client_id: str, **kwargs: Any) -> SearchDepartmentsResponse:
        return self._get(f'/clients/{client_id}/departments', kwargs)

    def get_gl_accounts(self, client_id: str, **kwargs: Any) -> SearchGLAccountsResponse:
        return self._get(f'/clients/{client_id}/glAccounts', kwargs)



# This file is generated from the MSB OpenAPI specification
# and is subject to changes. Do not edit this file directly


from enum import Enum
from typing import List, Literal, TypedDict


class APIServer(Enum):
    LAB = 'https://schoolbucks.msb-lab.com/msbpay/v2'
    QA = 'https://test.www.myschoolbucks.com/msbpay/v2'
    STAGING = 'https://staging.www.myschoolbucks.com/msbpay/v2'
    PRODUCTION = 'https://www.myschoolbucks.com/msbpay/v2'
    

MsbPayResult = Literal['unknown','success','error',]
CartStatus = Literal['pending','canceled','authorized','closed',]
PaymentTxnType = Literal['sale','credit','refund','reversal','preauth','postauth',]
PaymentType = Literal['sale','credit',]
OrderStatus = Literal['active','canceled','closed','declined','pending','review','void',]
PayApiEventType = Literal['ping','cart.authorized','cart.processed','cart.canceled','payment.settled','payment.authorized','payment.failed','payment.refunded',]

class MsbPayResponse(TypedDict):
    result: 'MsbPayResult'
    errors: List[str]
    
class MsbPayUser(TypedDict):
    defaultClientId: str
    developerAccountId: str
    firstName: str
    lastName: str
    email: str
    clients: List['MsbPayUserClient']
    
class MsbPayUserClient(TypedDict):
    isEnabled: bool
    clientId: str
    departmentId: str
    storeId: str
    paymentMethodId: str
    
class Cart(TypedDict):
    id: str
    status: 'CartStatus'
    createdDate: str
    lastUpdated: str
    redirectUrl: str
    returnToSiteUrl: str
    checkoutUrl: str
    allowDuplicatePayments: str
    loginPolicy: str
    checkoutStyle: str
    cartItems: 'CartItem'
    paymentPreauthPolicy: str
    webhookSubscriptions: List['WebhookSubscription']
    
class WebhookSubscription(TypedDict):
    notifyUrl: str
    type: 'PayApiEventType'
    isEnabled: bool
    subscriptionId: str
    
class CartItem(TypedDict):
    clientId: str
    departmentId: str
    storeId: str
    itemId: str
    itemName: str
    paymentMethodId: str
    quantity: str
    unitPrice: str
    displaySalesTaxRate: float
    salesTaxAmount: float
    studentName: str
    reference: str
    properties: List['ItemProperty']
    glAccountId: str
    
class CartOrder(TypedDict):
    clientId: str
    orderId: str
    date: str
    status: 'OrderStatus'
    recipientName: str
    recipientEmail: str
    billingAcctDesc: str
    tag: object
    storeId: str
    paymentMethodId: str
    cartOrderItems: List['CartOrderItem']
    
class CartOrderItem(TypedDict):
    orderItemId: str
    itemName: str
    itemId: str
    unitPrice: float
    quantity: int
    price: float
    userTxnFee: float
    clientTxnFee: float
    salesTax: float
    reference: str
    properties: List['ItemProperty']
    studentName: str
    
class CartPayment(TypedDict):
    id: str
    clientId: str
    cartId: str
    orderId: str
    payerName: str
    date: str
    txnType: 'PaymentTxnType'
    storeId: str
    paymentMethodId: str
    billingAcctDesc: str
    amount: float
    salesTax: float
    userTxnFee: float
    clientTxnFee: float
    confirmation: str
    batchDate: str
    batchNumber: str
    depositAmount: float
    settled: bool
    cartPaymentItems: List['CartPaymentItem']
    remainingRefundableAmount: float
    
class CartPaymentItem(TypedDict):
    paymentItemId: str
    itemName: str
    itemId: str
    unitPrice: float
    quantity: int
    price: float
    userTxnFee: float
    clientTxnFee: float
    salesTax: float
    reference: str
    glAccountId: str
    studentName: str
    properties: List['ItemProperty']
    
class CartRefundItem(TypedDict):
    departmentId: str
    itemId: str
    itemName: str
    quantity: str
    unitPrice: str
    salesTax: str
    studentName: str
    glAccountId: str
    reference: str
    properties: List['ItemProperty']
    
class ItemProperty(TypedDict):
    name: str
    value: str
    displayResponse: str
    
class ClientProperty(TypedDict):
    id: str
    name: str
    
class ListPaging(TypedDict):
    page: int
    prev: str
    self: str
    next: str
    total: int
    
class GetMsbPayUserResponse(MsbPayResponse):
    user: 'MsbPayUser'
    
class GetCartsResponse(MsbPayResponse):
    carts: List['Cart']
    meta: 'ListPaging'
    
class GetCartResponse(MsbPayResponse):
    cart: 'Cart'
    
class CreateUpdateCartResponse(MsbPayResponse):
    cartId: str
    
class ProcessCartResponse(MsbPayResponse):
    resultCodes: List[str]
    cartId: str
    paymentIds: List[str]
    
class BooleanResponse(MsbPayResponse):
    responseBoolean: bool
    
class GetCartOrdersResponse(MsbPayResponse):
    cartOrders: List['CartOrder']
    
class CartPaymentRefundResponse(MsbPayResponse):
    paymentId: str
    cartId: str
    
class SearchPaymentsResponse(MsbPayResponse):
    cartPayments: List['CartPayment']
    meta: 'ListPaging'
    
class SearchPaymentMethodsResponse(MsbPayResponse):
    paymentMethods: List['ClientProperty']
    meta: 'ListPaging'
    
class SearchStoresResponse(MsbPayResponse):
    stores: List['ClientProperty']
    meta: 'ListPaging'
    
class SearchDepartmentsResponse(MsbPayResponse):
    departments: List['ClientProperty']
    meta: 'ListPaging'
    
class SearchGLAccountsResponse(MsbPayResponse):
    glaccounts: List['ClientProperty']
    meta: 'ListPaging'



from datetime import datetime


DATE_FORMAT = '%Y-%m-%dT%H:%M:%S.%f%Z'


def to_msbpay_string(dt: datetime) -> str:
    return datetime.strftime(dt, DATE_FORMAT)


def from_msbpay_string(dt: str) -> datetime:
   return datetime.strptime(dt, DATE_FORMAT)
